package svc

import (
	"ark-permission/internal/config"
	"ark-permission/internal/domain/repository"
	domainUseCase "ark-permission/internal/domain/usecase"
	"ark-permission/internal/lib/required"
	"ark-permission/internal/model"
	repo "ark-permission/internal/repository"
	"ark-permission/internal/usecase"
	ers "code.30cm.net/wanderland/library-go/errors"
	"code.30cm.net/wanderland/library-go/errors/code"
	"context"
	"github.com/zeromicro/go-zero/core/stores/redis"
	"github.com/zeromicro/go-zero/core/stores/sqlx"
)

type ServiceContext struct {
	Config config.Config

	Validate       required.Validate
	Redis          redis.Redis
	TokenRedisRepo repository.TokenRepository
	Permission     model.PermissionModel
	PolicyAgent    domainUseCase.OpaUseCase
}

func NewServiceContext(c config.Config) *ServiceContext {
	newRedis, err := redis.NewRedis(c.RedisCluster, redis.Cluster())
	if err != nil {
		panic(err)
	}

	ers.Scope = code.CloudEPPermission
	sqlConn := sqlx.NewMysql(c.DB.DsnString)

	pa, err := usecase.NewOpaUseCase(usecase.OpaUseCaseParam{})
	if err != nil {
		panic(err)
	}
	// TODO policy 權限還要再組合過，我的角度會把 UID 當成一種 RoleID 這樣就可以針對每一個人克制權限，，初期也可以使用最簡安的來做統一，再想一下
	err = pa.LoadPolicy(context.Background(), []domainUseCase.Policy{})
	if err != nil {
		panic(err)
	}

	return &ServiceContext{
		Config:   c,
		Validate: required.MustValidator(),
		Redis:    *newRedis,
		TokenRedisRepo: repo.NewTokenRepository(repo.TokenRepositoryParam{
			Store: newRedis,
		}),
		Permission:  model.NewPermissionModel(sqlConn),
		PolicyAgent: pa,
	}
}
